package org.seasar.extension.dataset.impl;

import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.Date;
import java.util.TimeZone;

import jxl.BooleanCell;
import jxl.Cell;
import jxl.CellType;
import jxl.DateCell;
import jxl.NumberCell;
import jxl.Sheet;
import jxl.Workbook;
import jxl.read.biff.BiffException;

import org.seasar.extension.dataset.DataReader;
import org.seasar.extension.dataset.DataRow;
import org.seasar.extension.dataset.DataSet;
import org.seasar.extension.dataset.DataSetConstants;
import org.seasar.extension.dataset.DataTable;
import org.seasar.extension.dataset.JXLRuntimeException;
import org.seasar.framework.exception.IORuntimeException;
import org.seasar.framework.util.Base64Util;
import org.seasar.framework.util.ResourceUtil;
import org.seasar.framework.util.StringUtil;
import org.seasar.framework.util.TimestampConversionUtil;

/**
 * @author taichi S.
 */
public class JXlsReader implements DataReader, DataSetConstants {

	private DataSet dataSet_;
	private Workbook workbook;

	public JXlsReader(String path) {
		this(ResourceUtil.getResourceAsStream(path));
	}

	public JXlsReader(InputStream in) {
		try {
		    workbook = Workbook.getWorkbook(in);
		} catch(BiffException ex) { 
		    throw new JXLRuntimeException(ex);
		} catch (IOException ex) {
			throw new IORuntimeException(ex);
		}
		dataSet_ = new DataSetImpl();
		String[] names = workbook.getSheetNames();
		for (int i = 0; i < workbook.getNumberOfSheets(); ++i) {
			createTable(names[i], workbook.getSheet(i));
		}
	}

	/**
	 * @see org.seasar.extension.dataset.DataReader#read()
	 */
	public DataSet read() {
		return dataSet_;
	}

	private DataTable createTable(String sheetName, Sheet sheet) {
		DataTable table = dataSet_.addTable(sheetName);
		int rowCount = sheet.getRows();
		if (rowCount >= 0) {
			setupColumns(table, sheet.getRow(0));
			if (rowCount > 0) {
				setupRows(table, sheet);
			}
		}
		return table;
	}

	private void setupColumns(DataTable table, Cell[] row) {
		for (int i = 0; i < row.length ; i++) {
			Cell nameCell = row[i];
			if (nameCell == null) {
				break;
			}
			String nameCellValue = nameCell.getContents();
		    if (nameCellValue.length() == 0) {
		      break;
		    }
		    table.addColumn(nameCellValue);
		}
	}

	private void setupRows(DataTable table, Sheet sheet) {
		for (int i = 1; i < sheet.getRows() ; i++) {
			Cell[] row = sheet.getRow(i);
			if (row == null) {
				break;
			}
			setupRow(table, row);
		}
	}

	private void setupRow(DataTable table, Cell[] row) {
		DataRow dataRow = table.addRow();
		for (int i = 0; i < table.getColumnSize() && i < row.length; i++) {
			Cell cell = row[i];
			Object value = getValue(cell);
			dataRow.setValue(i, value);
		}
	}

	public boolean isCellBase64Formatted(Cell cell) {
		return BASE64_FORMAT.equals(cell.getCellFormat().getFormat().getFormatString());
	}

	public Object getValue(Cell cell) {
		if (cell == null) {
			return null;
		}
		CellType type = cell.getType();
		if(CellType.DATE == type || CellType.DATE_FORMULA == type) {
		    Date date = ((DateCell) cell).getDate();
		    // Excelǂݎlɑ΂āAftHg^C][GMTƂ̍]KvׁB
		    date.setTime(date.getTime() - TimeZone.getDefault().getRawOffset());
		    return TimestampConversionUtil.toTimestamp(date);
		} else if(CellType.NUMBER == type || CellType.NUMBER_FORMULA == type) {
		    return new BigDecimal(((NumberCell) cell).getValue());
		} else if(CellType.BOOLEAN == type || CellType.BOOLEAN_FORMULA == type) {
		    return Boolean.valueOf(((BooleanCell) cell).getValue());
		} else if(CellType.LABEL == type || CellType.STRING_FORMULA == type) {
		    String s = cell.getContents();
			if (s != null) {
				s = StringUtil.rtrim(s);
			}
			if ("".equals(s)) {
				s = null;
			}
			if (isCellBase64Formatted(cell)) {
				return Base64Util.decode(s);
			}
			return s;
		} else {
		    return cell.getContents();
		}
	}
}
